body {
    font-family: sans-serif;
    background-color: #222222;
    color: #e0d0d0;
    padding-left: 16px;
    padding-right: 16px;
}
a {
    color: #aaaaff;
}
hr {
   border: 0;
   color: #111111;
   background-color: #111111;
   height: 2px;
   margin-top: 16px;
}
h3 {
    color: #ffffff;
    font-size: 16px;
}
img {
    border:0;
    margin:0;
}
.sidebar {
   padding:10px;
   vertical-align: top;
}
.alert {
    background-color:#aa0000;
    color:#ffffff;
    font-weight:bold;
    padding: 6px 10px;
    width:500px;
}